
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#pragma once

#include "ExcelClone.h"
#include "Sheets.h"
#include "DispatchHook.h"
#include <string>

class CApplication;



/*
	CWorkbook class Represents a Microsoft Excel workbook by using OpenOffice.
	It implements IWorkbook COM interface.
*/
class ATL_NO_VTABLE CWorkbook :
	public CComObjectRootEx<CComSingleThreadModel>,
    public CDispatchHook< IDispatchImpl< IWorkbook, &IID_IWorkbook, &LIBID_ExcelCloneLib, 1, 0> >
{
public:
    CWorkbook();
    ~CWorkbook();

    // add new workbook in external application etc.
    HRESULT Initialize(CApplication  *pApp);
	HRESULT Initialize(CApplication  *pApp, CComVariant vFileName);

    BEGIN_COM_MAP(CWorkbook)
        COM_INTERFACE_ENTRY(IWorkbook)
        COM_INTERFACE_ENTRY(IDispatch)
    END_COM_MAP()

public:
	/** 
	*	Returns a Sheets collection that represents all the sheets in the active workbook, 
	*	for an Application object. Returns a Sheets collection that represents all the sheets 
	*	in the specified workbook, for a Workbook object.
	**/
	STDMETHOD(get_Sheets)(ISheets **ppSheets); 
    STDMETHOD(get_Worksheets)(ISheets** ppSheets);

public:
    // internal methods
    CComPtr<IDispatch> getOODocument() const;
    void Close(std::wstring pFilename); // Close Workbook

private:
    CComPtr<IDispatch> m_pdDoc;

protected:
    CSheetsObject m_Sheets;
	CApplication *m_pThisApp;

	friend class CSheets;
};

typedef CComObject<CWorkbook> CWorkbookObject;
typedef CComPtr<CWorkbookObject> CWorkbookObjectPtr;
