
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


// Workbooks.cpp

#include "StdAfx.h"
#include "Workbooks.h"
#include "application.h"
#include "Range.h"

#include <autowrap.h>

#include <string>
#include <algorithm>

extern CWorkbooks	*g_pWorkbooks;
/*
	CWorkbooks class Represents A collection of all the Workbook objects that are currently 
	open in the Microsoft Excel application by using OpenOffice.
	It implements IWorkbooks COM interface.
*/
CWorkbooks::CWorkbooks()
: m_pThisApp(NULL)
{
	g_pWorkbooks = this;
	WRITE_LOG_R_W((L"CWorkbooks::CWorkbooks")); 
}

CWorkbooks::~CWorkbooks()
{
	WRITE_LOG_R_W((L"CWorkbooks::~CWorkbooks")); 
	// Once this object is destroyed, we save and close all workbooks opened.
}

//void CWorkbooks::CloseAllFiles()
//{
//    WRITE_LOG_R_W((L"CWorkbooks::CloseAllFiles()"));
//    if (!CApplication::s_bShowed)
//    {
//	    CWorkbookObjectPtr ptr;
//	    // for each Workbook
//	    wchar_t tcs[20];
//	    SYSTEMTIME st;
//	    for (std::vector<CWorkbookObjectPtr>::iterator i = m_WorkbookVector.begin(); 
//		    i!=m_WorkbookVector.end(); i++) 
//	    {
//		    // Get system date to create unique name 
//		    GetLocalTime (&st);
//			int nKf = 8;
//			if (st.wMonth < 10)
//				nKf = 7;
//		    _itow(st.wMonth * 1000000 + st.wDay*10000+ st.wYear, tcs, 10);
//		    tcs[nKf]='_';
//		    _itow(st.wHour*10000 + st.wMinute*100+ st.wSecond, tcs + nKf + 1, 10);
//            std::wstring sFileName = (std::wstring (L"file:///") + m_pThisApp->m_sPath + m_pThisApp->m_sName + std::wstring(tcs) + L".xls");
//            WRITE_LOG_R_W((L"File name: %s", sFileName.c_str()));
//		    // Output file name is "file:///" + Path to file + name + unique name + ".xls"
//		    (*i)->Close((std::wstring (L"file:///") + m_pThisApp->m_sPath + m_pThisApp->m_sName + std::wstring(tcs) + L".xls" ));
//		}
//	}
//	TerminateProcess(GetCurrentProcess(), 0);
//}

CWorkbook* CWorkbooks::GetActiveWorkbook()
{
	// Active Workbook is last added one.
	WRITE_LOG_R_W((L"CWorkbooks::GetActiveWorkbook"));
    return m_WorkbookVector.back();
}

STDMETHODIMP CWorkbooks::Add(VARIANT varTemplate, IWorkbook **ppWorkbook)
{
    WRITE_LOG_R_W((L"CWorkbooks::Add(...)"));

    if (ppWorkbook == NULL)
    {
        return E_POINTER;
    }
    *ppWorkbook = NULL;

    try
    {
        CWorkbookObject *pWorkbook = new CWorkbookObject(this);
        HRESULT hr = pWorkbook->Initialize(m_pThisApp);
        if (FAILED(hr))
        {
            WRITE_LOG_R_W((L"FILED"));
            delete pWorkbook;
            return hr;
        }
        
        m_WorkbookVector.push_back(pWorkbook);
        *ppWorkbook = pWorkbook;
		(*ppWorkbook)->AddRef();
        return S_OK;
    }
    catch (const std::bad_alloc&)
    {
        return E_OUTOFMEMORY;
    }
}

// NEW!!!
STDMETHODIMP CWorkbooks::_Open( 
    BSTR        Filename,
    VARIANT     UpdateLinks,
    VARIANT     ReadOnly,
    VARIANT     Format,
    VARIANT     Password,
    VARIANT     WriteResPassword,
    VARIANT     IgnoreReadOnlyRecommended,
    VARIANT     Origin,
    VARIANT     Delimiter,
    VARIANT     Editable,
    VARIANT     Notify,
    VARIANT     Converter,
    VARIANT     AddToMru,
    long        Lcid,
    IWorkbook **ppWorkbook
    )
{
   WRITE_LOG_R_W((L"CWorkbooks::Add(...)"));

    if (ppWorkbook == NULL)
    {
        return E_POINTER;
    }
    *ppWorkbook = NULL;

	wchar_t *ptr = Filename;
	while (*ptr != NULL) 
	{
		if (*ptr == L'\\')
			*ptr = L'/';
		else 
			if (*ptr == L':')
				*ptr = L'|';
		ptr++;
	}
	std::wstring sName = Filename;
	sName = L"file:///" + sName;
	AtlTrace("\n");
	AtlTrace(sName.c_str());

    try
    {
        CWorkbookObject *pWorkbook = new CWorkbookObject(this);
        HRESULT hr = pWorkbook->Initialize(m_pThisApp, sName.c_str());
        if (FAILED(hr))
        {
            delete pWorkbook;
            return hr;
        }
        
        m_WorkbookVector.push_back(pWorkbook);
        *ppWorkbook = pWorkbook;
		(*ppWorkbook)->AddRef();
        return S_OK;
    }
    catch (const std::bad_alloc&)
    {
        return E_OUTOFMEMORY;
    }

    return S_OK;
}

STDMETHODIMP CWorkbooks::Open(
    BSTR Filename, 
    VARIANT UpdateLinks, 
    VARIANT ReadOnly, 
    VARIANT Format, 
    VARIANT Password, 
    VARIANT WriteResPassword, 
    VARIANT IgnoreReadOnlyRecommended, 
    VARIANT Origin, 
    VARIANT Delimiter, 
    VARIANT Editable, 
    VARIANT Notify, 
    VARIANT Converter, 
    VARIANT AddToMru, 
    VARIANT Local, 
    VARIANT CorruptLoad, 
    long lcid, 
    IWorkbook** ppWorkbook)
{
    return _Open( 
        Filename,
        UpdateLinks,
        ReadOnly,
        Format,
        Password,
        WriteResPassword,
        IgnoreReadOnlyRecommended,
        Origin,
        Delimiter,
        Editable,
        Notify,
        Converter,
        AddToMru,
        lcid,
        ppWorkbook
        );
}


STDMETHODIMP CWorkbooks::Close(long lcid)
{
   WRITE_LOG_R_W((L"CWorkbooks::Close"));
    return S_OK;
}

// Get active range(cell) on active sheet, in active workbook
HRESULT CWorkbooks::GetActiveCell(IRange **ppRange)
{
	WRITE_LOG_R_W((L"CWorkbooks::GetActiveCell(...)"));
    if (ppRange == NULL)
    {
        return E_INVALIDARG;
    }
    //m_WorkbookVector
    CWorkbook *pWorkbook = GetActiveWorkbook();
    if ((pWorkbook == NULL) || (pWorkbook->getOODocument() == NULL))
    {
        return E_FAIL;
    }
    HRESULT hres;
    VARIANT vRes;
    IDispatch *pdCurrentSelection;
    hres = AutoWrap(DISPATCH_PROPERTYGET, &vRes, pWorkbook->getOODocument(), L"CurrentSelection", 0);
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FAIL"));
        return hres;
    }
    pdCurrentSelection = vRes.pdispVal;
    CComVariant vRow    = 0;
    CComVariant vColumn = 0;
    IDispatch *pCurrentCell;
    hres = AutoWrap(DISPATCH_METHOD, &vRes, pdCurrentSelection, L"getCellByPosition", 2, vRow, vColumn);
    pdCurrentSelection->Release();
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FAIL"));
        return hres;
    }
    pCurrentCell = vRes.pdispVal;
    CRangeObject *pRange = new CRangeObject; 
    if (pRange == NULL)
    {
        return E_FAIL;
    }
    hres = pRange->Initialize(pCurrentCell);
    pCurrentCell->Release();
    *ppRange = pRange;
    return hres;
}

// Inintialize function
void CWorkbooks::Initialize(CApplication* pdApp)
{
	WRITE_LOG_R_W((L"CWorkbooks::Initialize"));
	m_pThisApp = pdApp;

}
