
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#pragma once

#include "ExcelClone.h"
#include "InternalObject.h"
#include "Range.h"
#include "DispatchHook.h"


// CWorksheet

class CWorkbook;
/*
	CWorksheet class represents Excel worksheet object by using OpenOffice.
	It implements IWorksheet COM interface.
*/

class ATL_NO_VTABLE CWorksheet :
	public CComObjectRootEx<CComSingleThreadModel>,
	public CDispatchHook< IDispatchImpl< IWorksheet, &IID_IWorksheet, &LIBID_ExcelCloneLib, 1, 0> >
{
public:
    CWorksheet();
    ~CWorksheet();

    BEGIN_COM_MAP(CWorksheet)
        COM_INTERFACE_ENTRY(IWorksheet)
        COM_INTERFACE_ENTRY(IDispatch)
    END_COM_MAP()

public:

    STDMETHOD (get_Name) (BSTR *pbstrName);
    STDMETHOD (put_Name) (BSTR bstrName);   
	/*
		Returns a Range object that represents all the cells on the 
		worksheet (not just the cells that are currently in use).
	*/
    STDMETHOD (get_Cells)(IRange **ppRange);
    
	/*
		Returns a Range object that represents a cell or a range of cells.
	*/
    STDMETHOD (get_Range)(VARIANT Cell1, VARIANT Cell2, IRange **ppRange);
    
	/*
		Pastes the contents of the Clipboard onto the sheet.
	*/
    STDMETHOD (Paste)    (VARIANT Destination, VARIANT Link, long lcid);
protected:
    CRangeIntObject m_allCells;
    CWorkbook       *m_pParentWorkbook;
	IDispatch       *m_pdSheet;
public:
	void Initialize(CWorkbook *pParentWorkbook, IDispatch *pdSheet);
    
};

typedef CComObject<CWorksheet> CWorksheetObject;
typedef CComPtr<CWorksheetObject> CWorksheetObjectPtr;
