
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#include "stdafx.h"
#include "WordBasic.h"

#define DATASOURCE_NAME L"ClonerDataSource"
#define TEMPFILE_NAME   L"template.sxw"
#define RESULTFILE_NAME L"cloners_temp"

COOApplication CWordBasic::s_pOOApp;

/*
	CWordBasic class provides functionality of the MS Office WordBasic by implementing IWordBasic interface.
*/

CWordBasic::CWordBasic() :
	m_pOOWDoc (NULL)
{
    WRITE_LOG_R_W((L"CWordBasic::CWordBasic()"));
	CWordBasic::s_pOOApp.CreateApplication();
}

    
CWordBasic::~CWordBasic()
{
    WRITE_LOG_R_W((L"CWordBasic::~CWordBasic()"));
}


STDMETHODIMP CWordBasic::FileOpen( 
    VARIANT Name,
    VARIANT ConfirmConversions,
    VARIANT ReadOnly,
    VARIANT AddToMru,
    VARIANT PasswordDoc,
    VARIANT PasswordDot,
    VARIANT Revert,
    VARIANT WritePasswordDoc,
    VARIANT WritePasswordDot
    )
{
    WRITE_LOG_R_W((L"CWordBasic::FileOpen"));

    wchar_t *ptr = NULL;
    std::wstring sName;
    if (Name.vt == VT_BSTR) 
    {
        ptr = Name.bstrVal;
    } else 
    {
        if ((Name.vt == (VT_BSTR | VT_BYREF)) && (Name.pbstrVal != NULL))
        {
            ptr = *Name.pbstrVal;
        } else 
        {
            return E_INVALIDARG;
        }
    }
	while (*ptr != NULL) 
	{
		if (*ptr == L'\\')
			*ptr = L'/';
		else 
			if (*ptr == L':')
				*ptr = L'|';
		ptr++;
	}
    if (Name.vt == VT_BSTR) 
    {
        sName = Name.bstrVal;
    } else 
    {
        sName = *Name.pbstrVal;
        // TODO : do this part most universality
        sName += L".DOC";
    } 	
	sName = L"file:///" + sName;
	//MessageBoxW(NULL, sName.c_str(), NULL, 0);
	//app.CreateBlankDocument(COODocument::TypeWriterDocument, &doc);

    m_pOOWDoc = new COOWriterDocument();
	COODocument *pOODoc;

    WRITE_LOG_R_W((L"File name: %s", sName.c_str()));

	CWordBasic::s_pOOApp.FileOpen(sName.c_str(), &pOODoc, TRUE);	
	m_pOOWDoc->Attach(pOODoc);
    return S_OK;
}
    
STDMETHODIMP CWordBasic::MailMergeOpenDataSource( 
    VARIANT Name,
    VARIANT ConfirmConversions,
    VARIANT ReadOnly,
    VARIANT LinkToSource,
    VARIANT AddToMru,
    VARIANT PasswordDoc,
    VARIANT PasswordDot,
    VARIANT Revert,
    VARIANT WritePasswordDoc,
    VARIANT WritePasswordDot,
    VARIANT Connection,
    VARIANT SQLStatement,
    VARIANT SQLStatement1
    )
{
    WRITE_LOG_R_W((L"CWordBasic::MailMergeOpenDataSource"));

    std::wstring sPath = *Name.pbstrVal;

    WRITE_LOG_R_W((L"File name: %s", sPath.c_str()));

    int iIndex = (int) sPath.find_last_of(L'\\');
    if (iIndex != -1)
    {
        sPath.erase(iIndex, sPath.size());
    }
    CComVariant vDSName = DATASOURCE_NAME;
    HRESULT hres = CWordBasic::s_pOOApp.CreateTextDataSource(vDSName, sPath.c_str());
    return S_OK;
}

STDMETHODIMP CWordBasic::ToolsOptionsEdit( 
    VARIANT ReplaceSelection,
    VARIANT DragAndDrop,
    VARIANT AutoWordSelection,
    VARIANT InsForPaste,
    VARIANT Overtype,
    VARIANT SmartCutPaste,
    VARIANT AllowAccentedUppercase,
    VARIANT PictureEditor,
    VARIANT TabIndent
    )
{
    return S_OK;
}

STDMETHODIMP CWordBasic::FileNewDefault()
{
    WRITE_LOG_R_W((L"CWordBasic::FileNewDefault()"));
	m_pOOWDoc = new COOWriterDocument();
	COODocument *pOODoc;
	CWordBasic::s_pOOApp.CreateBlankDocument(COODocument::TypeWriterDocument, &pOODoc, TRUE);	
	m_pOOWDoc->Attach(pOODoc);
    return S_OK;
}

STDMETHODIMP CWordBasic::FormFieldOptions( 
    VARIANT Entry,
    VARIANT Exit,
    VARIANT Name,
    VARIANT Enable,
    VARIANT TextType,
    VARIANT TextWidth,
    VARIANT TextDefault,
    VARIANT TextFormat,
    VARIANT CheckSize,
    VARIANT CheckWidth,
    VARIANT CheckDefault,
    VARIANT Type,
    VARIANT OwnHelp,
    VARIANT HelpText,
    VARIANT OwnStat,
    VARIANT StatText
    )
{
    WRITE_LOG_R_W((L"CWordBasic::FormFieldOptions(...)"));
	if (m_pOOWDoc == NULL)
		return E_FAIL;
	return m_pOOWDoc->SetCurrentBookmarkProperties(Name, TextDefault);
}

STDMETHODIMP CWordBasic::InsertFormField( 
    VARIANT Entry,
    VARIANT Exit,
    VARIANT Name,
    VARIANT Enable,
    VARIANT TextType,
    VARIANT TextWidth,
    VARIANT TextDefault,
    VARIANT TextFormat,
    VARIANT CheckSize,
    VARIANT CheckWidth,
    VARIANT CheckDefault,
    VARIANT Type,
    VARIANT OwnHelp,
    VARIANT HelpText,
    VARIANT OwnStat,
    VARIANT StatText
    )
{
    WRITE_LOG_R_W((L"CWordBasic::InsertFormField"));
	if (m_pOOWDoc == NULL)
		return E_FAIL;
	return m_pOOWDoc->InsertInputField(Name, Name, TextDefault);
}

STDMETHODIMP CWordBasic::MailMergeToDoc()
{
    //DATASOURCE_NAME

    //TODO: MailMergeToDoc needs to refactoring

    WRITE_LOG_R_W((L"CWordBasic::MailMergeToDoc()"));

    wchar_t szTempPath[MAX_PATH];
    // read keys from ini file
    ::GetTempPath(MAX_PATH, szTempPath);
    std::wstring sTempPath = szTempPath;
    // Generate temp directory 
    DWORD dwTickCount = GetTickCount();
    wchar_t szTickCount[MAX_PATH];
    _itow(dwTickCount, szTickCount, 10);
    sTempPath += szTickCount;

    //
    ::CreateDirectory(sTempPath.c_str(), NULL);

    HRESULT hres = m_pOOWDoc->BindToDatasource(DATASOURCE_NAME, L"WordMail");
    CComVariant vParam;
    vParam.vt = VT_ERROR;
    std::wstring sPath;
    COOApplication::ConvertPathToUrl(sTempPath, sPath);
    std::wstring sTmp = sPath + TEMPFILE_NAME;
    hres = m_pOOWDoc->m_pDoc->SaveFile(sTmp.c_str(), vParam);

// TODO : Replace hardcoded table name "WordMail"
    sTmp = sTempPath + TEMPFILE_NAME;
    
    hres = CWordBasic::s_pOOApp.MailMerge(DATASOURCE_NAME, sTmp.c_str(), sTempPath.c_str(), L"WordMail", RESULTFILE_NAME);

    
    std::vector<std::wstring> data;
    std::wstring sSearchPath = sTempPath + RESULTFILE_NAME;
    std::wstring sTempFolder = sTempPath;

	WIN32_FIND_DATA FileData;
	HANDLE hSearch; 

	BOOL fFinished = FALSE; 
	sSearchPath += L"*";
	hSearch = FindFirstFile(sSearchPath.c_str(), &FileData); 
	std::string s;
	if (hSearch != INVALID_HANDLE_VALUE) 
        data.push_back(sTempFolder + FileData.cFileName);
	else 
		return S_OK;
	while (!fFinished) 
    { 
		if (!FindNextFile(hSearch, &FileData)) 
		{
			if (GetLastError() == ERROR_NO_MORE_FILES) 
			{
				fFinished = TRUE; 
			} 
		} 
		else 
		{
            data.push_back(sTempFolder + FileData.cFileName);
		}
	} 
    CWordBasic::s_pOOApp.CombineDocuments(data, L"" /*Not used */, TRUE);

    std::vector<std::wstring>::iterator iEnd = data.end();
    std::vector<std::wstring>::iterator iCurrent = data.begin();
    for (; iCurrent != iEnd; iCurrent++)
    {
        DeleteFile((*iCurrent).c_str());
    }
    sTmp = sTempFolder + TEMPFILE_NAME;
    DeleteFile(sTmp.c_str());
    CComVariant vTemp = 2;
    DocClose(vTemp);
   
    return S_OK;//hres
}

STDMETHODIMP CWordBasic::ShowAll(VARIANT On, short *pResult)
{
    if (pResult == NULL)
    {
        return E_POINTER;
    }

    return S_OK;
}

STDMETHODIMP CWordBasic::AppActivate(BSTR WindowName, VARIANT Immediate)
{
    return S_OK;
}

STDMETHODIMP CWordBasic::NextField(short *pResult)
{
    WRITE_LOG_R_W((L"CWordBasic::NextField(...)"));
    if (pResult == NULL)
    {
        return E_POINTER;
	}
	//static DWORD dwBookmarksCount = 0xFFFFFFFF;
	//if (m_pOOWDoc == NULL)
	//	return E_FAIL;
	//HRESULT hres;
	//BOOL  bNewResult = FALSE;
	//if (dwBookmarksCount  == 0 || dwBookmarksCount  == dwBookmarksCount  )
	//{
	//	bNewResult = TRUE;
	//	if (0xFFFFFFFF)
	//	{
	//		bNewResult = FALSE;
	//	}
	//	hres = m_pOOWDoc->GetBookmarkscount(dwBookmarksCount);
	//	if (hres != S_OK)
	//	{
	//		return hres;
	//	}
	//}
	//if (!bNewResult)
	//{
	//	//*pResult = 1;
	//	//dwBookmarksCount--;
	//	*pResult = dwBookmarksCount;
	//} 
	//else 
	//{
	//	*pResult = 0;
	//}
	if (m_pOOWDoc == NULL)
		return E_FAIL;
	
	DWORD dwCurrIndex		= m_pOOWDoc->GetCurrentBookmarkIndex();
	DWORD dwBookmarksCount;
	m_pOOWDoc->GetBookmarksCount(dwBookmarksCount);
	

	if (dwCurrIndex == dwBookmarksCount)
	{
		*pResult = 0;
	} 
	else
	{
		*pResult = 1;
	}
	return m_pOOWDoc->SelectBookmarkByIndex(dwCurrIndex + 1);
}

STDMETHODIMP CWordBasic::AppShow(VARIANT WindowName)
{
    return S_OK;
}

STDMETHODIMP CWordBasic::GetFormResultStr(BSTR BookmarkName, VARIANT *pResult)
{
    WRITE_LOG_R_W((L"CWordBasic::GetFormResultStr(...)"));

    if (pResult == NULL)
    {
        return E_POINTER;
    }
	if (m_pOOWDoc == NULL)
		return E_FAIL;
	CComVariant vRes;
	HRESULT hres = m_pOOWDoc->GetBookmarkText(BookmarkName, &vRes);

    WRITE_LOG_R_W((L"Book mark text: %s", BookmarkName));

    if (hres != S_OK)
	{
		return hres;
	}
	*pResult = vRes;
	return S_OK;

}

STDMETHODIMP CWordBasic::SetFormResult( 
    BSTR    BookmarkName,
    VARIANT Result,
    VARIANT DefaultResult
    )
{
    
    WRITE_LOG_R_W((L"CWordBasic::SetFormResult(...)"));
    WRITE_LOG_R_W((L"Bookmark name: %s", BookmarkName));
    if (Result.vt == VT_BSTR)
    {
        WRITE_LOG_R_W((L"Bookmark Result: %s", Result.bstrVal));
    }

	if (m_pOOWDoc == NULL)
		return E_FAIL;
	return m_pOOWDoc->SetBookmarkText(BookmarkName, Result);
}

STDMETHODIMP CWordBasic::GetSelStartPos(long *pSelStartPos)
{
    WRITE_LOG_R_W((L"CWordBasic::GetSelStartPos(...)"));
    
    if (pSelStartPos == NULL)
    {
        return E_POINTER;
    }

    *pSelStartPos = 0;

    return S_OK;
}

STDMETHODIMP CWordBasic::SetSelRange(long Pos1, long Pos2)
{
    WRITE_LOG_R_W((L"CWordBasic::SetSelRange(...)"));
	return m_pOOWDoc->SelectBookmarkByIndex(0);
}

STDMETHODIMP_(void)CWordBasic::DocClose (VARIANT Save)
{
    WRITE_LOG_R_W((L"CWordBasic::DocClose(...)"));
    
    if (Save.intVal == 2)
    {
        m_pOOWDoc->m_pDoc->Close();
        delete m_pOOWDoc;
        m_pOOWDoc = NULL;
    }
}

