
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#include "OOApplication.h"
#include "OOWriterdocument.h"
#include "PropertyValue.h"

#include <AutoWrap.h>

/*
    COOAplication is a bridge to the Open Office. It provides a functionality similar to Word Application class
    by utilizing OpenOffice API.
*/

COOApplication::COOApplication(void):
    m_pdDesktop(NULL),
    m_pdOOApp(NULL),
    m_pdDatabaseContext(NULL),
    m_pdActiveConnection(NULL)
{
}

COOApplication::~COOApplication(void)
{

}
HRESULT COOApplication::CreateApplication()
{
    CLSID clsid;  
    if ((m_pdDesktop != NULL) && (m_pdOOApp != NULL))
        return E_FAIL;
    HRESULT hres = CLSIDFromProgID(L"com.sun.star.ServiceManager", &clsid); 
    if(FAILED(hres)) 
        return hres;
    // Start server and get IDispatch... 
    hres = CoCreateInstance(clsid, NULL, CLSCTX_LOCAL_SERVER, IID_IDispatch, (void **)&m_pdOOApp); 
    if(FAILED(hres))
    {
        return hres; 
    }

    // Get Desktop and its assoc. IDispatch... 
    CComVariant result;
    CComVariant param1(L"com.sun.star.frame.Desktop"); 
    hres = AutoWrap(DISPATCH_METHOD, &result, m_pdOOApp, L"createInstance", 1, param1); 
    if (FAILED(hres)) 
    {
        m_pdOOApp->Release();
        return hres;
    }
    m_pdDesktop= result.pdispVal; 
    m_pdDesktop->AddRef(); 
    return S_OK;
}

HRESULT COOApplication::DestroyApplication()
{
    if (m_pdDesktop != NULL)
        m_pdDesktop->Release();
    if (m_pdOOApp != NULL)
        m_pdOOApp->Release();
    return S_OK;
}

HRESULT COOApplication::Attach(IDispatch  *pdOOApp, IDispatch  *pdDesktop)
{
    
    if ((pdDesktop != NULL) && (pdOOApp != NULL))
    {
        this->Detach();
        m_pdDesktop = pdDesktop;
        m_pdDesktop->AddRef();
        m_pdOOApp = pdOOApp;
        m_pdOOApp->AddRef();
        return S_OK;
    } else 
        return E_INVALIDARG;
}
HRESULT COOApplication::Attach(COOApplication *pApp)
{
    if (pApp == NULL)
    {
        return E_INVALIDARG;
    }
    return this->Attach(pApp->m_pdOOApp, pApp->m_pdDesktop);
}

HRESULT COOApplication::Detach()
{
    if (m_pdDesktop != NULL)
        m_pdDesktop->Release();
    if (m_pdOOApp != NULL)
        m_pdOOApp->Release();
    return S_OK;
}


HRESULT COOApplication::GetDispatchPropertyValue(IDispatch** pIDispatch)
{
    // there are many of the Open Office functions use "com.sun.star.beans.PropertyValue",
    // using this method
    VARIANT res;
    CComVariant cvStr(L"com.sun.star.beans.PropertyValue");
    HRESULT hres = AutoWrap(DISPATCH_METHOD, &res, m_pdOOApp, L"Bridge_GetStruct", 1, cvStr);
    if (hres == S_OK) 
    {
        *pIDispatch = res.pdispVal;
        (*pIDispatch)->AddRef();
        
    }
    else 
        *pIDispatch = NULL;
    return hres;
}


HRESULT COOApplication::CreateBlankDocument (const std::wstring &sDocumentType, COODocument **pDocument, BOOL bShowed)
{
    CComVariant ResultDoc;
    CComVariant p1 (L"Hidden");

    CComVariant params[4]; 
    VariantInit(&params[0]);            //private:factory/scalc
    params[0] = sDocumentType.c_str();  // Type of created document
    params[1] = L"_blank";              // Template
    params[2] = 1;                      // Another params count
    if (!bShowed)

    {
        CPropertyValue PropValue(m_pdOOApp);
        VARIANT pp;
        VariantInit (&pp);
        pp.vt = VT_BOOL;
        pp.boolVal = (VARIANT_TRUE);

        PropValue.SetPropertyValue(p1, pp);

        SAFEARRAY FAR* pPropVals = SafeArrayCreateVector( VT_DISPATCH, 0, 1 );
        long ix = 0; 
        HRESULT hres = SafeArrayPutElement( pPropVals, &ix, PropValue.GetValueDispatch() ); 
        if (FAILED (hres)) 
        {
            return hres;
        }
        VariantInit(&params[3]); 
        params[3].vt = VT_DISPATCH | VT_ARRAY;
        params[3].parray = pPropVals;

    }
    else
    {
        params[3] = NULL;
        params[3].vt =  VT_ARRAY | VT_VARIANT; 
    }
    HRESULT hres;
    hres = AutoWrap(DISPATCH_METHOD, &ResultDoc, m_pdDesktop, L"loadComponentFromURL", 4, params[3], params[2], params[1], params[0]); 
    if (FAILED(hres))
        return hres;
    IDispatch *pdDoc = ResultDoc.pdispVal; 
    if (pDocument != NULL) 
    {
        *pDocument = new COODocument(this, pdDoc);
        if (pDocument == NULL) 
        {

            pdDoc->Release();
            return E_FAIL;
        }
    }
    return hres;
}

HRESULT COOApplication::FileOpen (CComVariant vName, COODocument **pDocument, BOOL bShowed)
{
//Set objDocument= objDesktop.loadComponentFromURL("file:///C|/Documents and Settings/bayda/Desktop/Prompt.dot", "_blank", 0, args)
    HRESULT hres;
    if (m_pdDesktop == NULL)
    {
        hres = this->CreateApplication();
        if (hres != S_OK)
            return hres;
    }
    CComVariant res;    
    IDispatch *dpv = NULL;
    CComVariant p1 (L"Hidden");

    CComVariant ResultDoc;
    CComVariant params[4]; 
    VariantInit(&params[0]);            //private:factory/scalc
    params[0] = vName;                  // Type of created document
    params[1] = L"_blank";              // Template
    params[2] = 1;                      // Another params count
    if (!bShowed)

    {
        CPropertyValue PropValue(m_pdOOApp);
        VARIANT pp;
        VariantInit (&pp);
        pp.vt = VT_BOOL;
        pp.boolVal = (VARIANT_TRUE);

        PropValue.SetPropertyValue(p1, pp);

        SAFEARRAY FAR* pPropVals = SafeArrayCreateVector( VT_DISPATCH, 0, 1 );
        long ix = 0; 
        hres = SafeArrayPutElement( pPropVals, &ix, PropValue.GetValueDispatch() ); 
        if (FAILED (hres)) 
        {
            return hres;
        }
        VariantInit(&params[3]); 
        params[3].vt = VT_DISPATCH | VT_ARRAY;
        params[3].parray = pPropVals;

    }
    else
    {
        params[3] = NULL;
        params[3].vt =  VT_ARRAY | VT_VARIANT; 
    }
    hres = AutoWrap(DISPATCH_METHOD, &ResultDoc, m_pdDesktop, L"loadComponentFromURL", 4, params[3], params[2], params[1], params[0]);  
    if (hres != S_OK)
        return hres;
    IDispatch *pdDoc = ResultDoc.pdispVal; 
    if (pDocument != NULL) 
    {
        *pDocument = new COODocument(this, pdDoc);
        if (pDocument == NULL) 
        {
            pdDoc->Release();
            return E_FAIL;
        }
    }
    return hres;
}

HRESULT COOApplication::CreateTextDataSource   (CComVariant vDSName,
                                                CComVariant vPathName)
{
    HRESULT hres;

    //Set objDatabaseContext  = objServiceManager.createInstance("com.sun.star.sdb.DatabaseContext") 

    if (m_pdDatabaseContext == NULL)
    {
        hres = CreateDatabaseContext();
        if (hres != S_OK)
        {
            return E_FAIL;
        }
    }
    BOOL bIsExist;
    hres = IsExist(vDSName, bIsExist);
    if (hres != S_OK)
    {
        return E_FAIL;
    }
    if (bIsExist)
    {
        hres = RemoveDataSource(vDSName);
        if (hres != S_OK)
        {
            return E_FAIL;
        }
    }

// Set objDataSource       = objDatabaseContext.createInstance()
    IDispatch   *pDataSource;
    VARIANT     vRes;
    hres = AutoWrap(DISPATCH_METHOD, &vRes, m_pdDatabaseContext, L"createInstance", 0);
    if (hres != S_OK)
    {
        return hres;
    }
    pDataSource = vRes.pdispVal;

// objDatabaseContext.registerObject "TestData", objDataSource
    CComVariant vParam;
    vParam.vt       = VT_DISPATCH;
    vParam.pdispVal = pDataSource;

    hres = AutoWrap(DISPATCH_METHOD, &vRes, m_pdDatabaseContext, L"registerObject", 2, vParam, vDSName);
    VariantInit(&vParam);
// Setting datasource properties
    std::wstring sPath = vPathName.bstrVal;
    std::wstring sUrl;
    COOApplication::ConvertPathToDSUrl (sPath, sUrl);

    hres = COOApplication::SetURL(pDataSource, sUrl.c_str());
    if (hres != S_OK)
    {
        pDataSource->Release();
        return hres;
    }
// Set Info (AllDelimeters)
//Dim args(2)
//Set args(0) = objDataSource.Bridge_GetStruct("com.sun.star.beans.PropertyValue")
//args(0).Name    = "HeaderLine"
//args(0).Value   = True
//'objDataSource.Info = args
//
//Set args(1) = objDataSource.Bridge_GetStruct("com.sun.star.beans.PropertyValue")
//args(1).Name    = "ThousandDelimiter"
//args(1).Value   = "_"
//'objDataSource.Info = args
//
//
//Set args(2) = objDataSource.Bridge_GetStruct("com.sun.star.beans.PropertyValue")
//args(2).Name    = "FieldDelimiter"
//args(2).Value   = ","
//objDataSource.Info = args

    IDispatch *pdProperty[4];
    CComVariant param;
// Property 1
    hres = GetDispatchPropertyValue (&pdProperty[0]);
    if (hres == S_OK)
    {
        param.vt        = VT_BOOL;
        param.boolVal   = VARIANT_TRUE;
        hres = SetPropertyValue(pdProperty[0], L"HeaderLine", param);
        if (hres != S_OK)
        {
            pdProperty[0]->Release();
            pdProperty[0] = NULL;
        }
    } else 
    {
        pdProperty[0] = NULL;
    }
// Property 2
    hres = GetDispatchPropertyValue (&pdProperty[1]);
    if (hres == S_OK)
    {
        param = L"_";
        hres = SetPropertyValue(pdProperty[1], L"ThousandDelimiter", param);
        if (hres != S_OK)
        {
            pdProperty[1]->Release();
            pdProperty[1] = NULL;
        }

    } else 
    {
        pdProperty[1] = NULL;
    }

// Property 3
    hres = GetDispatchPropertyValue (&pdProperty[2]);
    if (hres == S_OK)
    {
        param = L",";
        hres = SetPropertyValue(pdProperty[2], L"FieldDelimiter", param);
        if (hres != S_OK)
        {
            pdProperty[2]->Release();
            pdProperty[2] = NULL;
        }

    } else 
    {
        pdProperty[2] = NULL;
    }
// Property 4
    hres = GetDispatchPropertyValue (&pdProperty[3]);
    if (hres == S_OK)
    {
        param = L"txt";
        hres = SetPropertyValue(pdProperty[3], L"Extension", param);
        if (hres != S_OK)
        {
            pdProperty[3]->Release();
            pdProperty[3] = NULL;
        }

    } else 
    {
        pdProperty[2] = NULL;
    }
// set this propertyes
    SAFEARRAY FAR* pPropVals = SafeArrayCreateVector( VT_DISPATCH, 0, 4 );
    long ix = 0; 
    for (int i = 0; i < 4; i++)
    {
        ix = i;
        SafeArrayPutElement(pPropVals, &ix, pdProperty[i]);
    }
    VARIANT vArrayParam;
    vArrayParam.vt = VT_ARRAY | VT_DISPATCH;
    vArrayParam.parray = pPropVals;
    hres = AutoWrap(DISPATCH_PROPERTYPUT, &vRes, pDataSource, L"Info", 1, vArrayParam);
    for (i = 0; i< 4; i++)
    {
        if (pdProperty[i] != NULL)
            pdProperty[i]->Release();
    }
//objDataSource.URL = "sdbc:flat:file:///c:/temp"
    if (hres != S_OK)
    {
        SafeArrayDestroy(pPropVals);
        pDataSource->Release();
        return hres;
    }

    VariantInit(&vArrayParam);
    
//    VariantInit(&vArrayParam);
    SafeArrayDestroy(pPropVals);
    hres = AutoWrap(DISPATCH_METHOD, &vRes, pDataSource, L"getConnection", 2, L"", L"");
    if (hres == S_OK)
    {
        if (m_pdActiveConnection != NULL)
        {
            m_pdActiveConnection->Release();
        }
        m_pdActiveConnection = vRes.pdispVal;
        VariantInit(&vRes);
    }
    pDataSource->Release();
    return hres;
}

HRESULT COOApplication::SetURL (IDispatch *pDatasource, CComVariant vUrl)
{
    HRESULT hres;
    VARIANT vRes;
    if (pDatasource == NULL)
    {
        return E_INVALIDARG;
    }
    hres = AutoWrap(DISPATCH_PROPERTYPUT, &vRes, pDatasource, L"URL", 1, vUrl);
    return hres;
}
HRESULT COOApplication::RemoveDataSource (CComVariant vDSName)
{
    if (m_pdDatabaseContext == NULL)
    {
        return E_FAIL;
    }
    HRESULT hres;
    VARIANT vRes;
    hres = AutoWrap(DISPATCH_METHOD, &vRes, m_pdDatabaseContext, L"revokeObject", 1, vDSName);
    return hres;
}
HRESULT COOApplication::IsExist    (CComVariant vDSName, BOOL &bResult)
{
    if (m_pdDatabaseContext == NULL)
    {
        return E_FAIL;
    }
    HRESULT hres;
    VARIANT vRes;
    hres = AutoWrap(DISPATCH_METHOD, &vRes, m_pdDatabaseContext, L"hasByName", 1, vDSName);
    if (hres == S_OK)
    {
        if (vRes.boolVal == VARIANT_FALSE)
        {
            bResult = FALSE;
        } 
        else 
        {
            bResult = TRUE;
        }
    }
    return hres;
}
HRESULT COOApplication::CreateDatabaseContext()
{
    if (m_pdDatabaseContext != NULL)
    {
        return S_OK;
    }
    if (m_pdOOApp == NULL)
    {
        return E_FAIL;
    }
    HRESULT hres;
    VARIANT vRes;
    CComVariant vClassName = L"com.sun.star.sdb.DatabaseContext";
    hres = AutoWrap(DISPATCH_METHOD, &vRes, m_pdOOApp, L"createInstance", 1, vClassName);
    if (hres == S_OK)
    {
        m_pdDatabaseContext = vRes.pdispVal; 
    }
    return hres;
}

/*
    Convert Windows path to URL 
*/
void COOApplication::ConvertPathToUrl            (const std::wstring &sPath, std::wstring &sURL)
{
    std::wstring            sSource = sPath;
    std::wstring::iterator iEnd     = sSource.end();
    std::wstring::iterator iCursor  = sSource.begin();
    for (;iCursor != iEnd; iCursor++)
    {
        
        if (*iCursor == L'\\')
        {
            *iCursor = L'/';
        }
    }
    sURL = L"file:///" + sSource;
}

/*
    Convert Windows path to database URL 
*/
void COOApplication::ConvertPathToDSUrl          (const std::wstring &sPath, std::wstring &sURL)
{
    std::wstring            sSource = sPath;
    std::wstring::iterator iEnd     = sSource.end();
    std::wstring::iterator iCursor  = sSource.begin();
    for (;iCursor != iEnd; iCursor++)
    {
        if (*iCursor == L'\\')
        {
            *iCursor = L'/';
        }
        else if (*iCursor == L':')
        {
            *iCursor = L'|';
        }
    }
    sURL = L"sdbc:flat:file:///" + sSource;
}

HRESULT COOApplication::MailMerge  (CComVariant vDSName,
                                    CComVariant vTemplateName,
                                    CComVariant vDestinationName,
                                    CComVariant vTableName,
                                    CComVariant vResultFileName)
{
    HRESULT hres;
    IDispatch *pMailmerge;
    hres = CreateMailMergeService (&pMailmerge);
    if (hres != S_OK)
    {
        return hres;
    }
    CComVariant vName;
    CComVariant vValue;

//objMailMerge.DataSourceName = "TestData"
    hres = SetMailMergeProperty(pMailmerge, L"DataSourceName", vDSName);
//objMailMerge.ActiveConnection = objConnection 

    std::wstring sPath;
    ConvertPathToUrl(vTemplateName.bstrVal, sPath);
    hres = SetMailMergeProperty(pMailmerge, L"DocumentURL", sPath.c_str());
//objMailMerge.CommandType    = 0
    hres = SetMailMergeProperty(pMailmerge, L"CommandType", 0);
//objMailMerge.Command        = "Wordmail"
    hres = SetMailMergeProperty(pMailmerge, L"Command", vTableName);
//objMailMerge.OutputType     = 2
    hres = SetMailMergeProperty(pMailmerge, L"OutputType", 2);
//objMailMerge.OutputURL      = "file:///c|/temp/"
    sPath = vDestinationName.bstrVal;
    std::wstring sUrl;
    COOApplication::ConvertPathToUrl(sPath, sUrl);
    hres = SetMailMergeProperty(pMailmerge, L"OutputURL", sUrl.c_str());
//objMailMerge.FileNamePrefix = "auto"
    hres = SetMailMergeProperty(pMailmerge, L"FileNamePrefix", vResultFileName);
    if (m_pdActiveConnection != NULL)
    {
        CComVariant vConnection;
        vConnection.vt = VT_DISPATCH;
        vConnection.pdispVal = m_pdActiveConnection;
        hres = SetMailMergeProperty(pMailmerge, L"ActiveConnection", vConnection);
        VariantInit(&vConnection);
    }
    
    CComVariant vParam;
    vParam.vt = VT_ARRAY | VT_VARIANT;
    vParam.parray = NULL;
    VARIANT vRes;
    hres = AutoWrap(DISPATCH_METHOD, &vRes, pMailmerge, L"Execute", 1, vParam);
    pMailmerge->Release();
    return hres;
}

HRESULT COOApplication::SetMailMergeProperty (IDispatch   *pdMergeObject,
                                              CComVariant vName,
                                              CComVariant vValue)
{
    if (pdMergeObject == NULL)
    {
        return E_FAIL;
    }
    HRESULT hres;
    VARIANT vRes;
    hres = AutoWrap(DISPATCH_PROPERTYPUT, &vRes, pdMergeObject, vName.bstrVal, 1, vValue);
    return hres;
}

HRESULT COOApplication::CreateMailMergeService      (IDispatch **ppResult)
{
    if (m_pdOOApp == NULL)
    {
        return E_FAIL;
    }
    CComVariant vParam1 = L"com.sun.star.text.MailMerge";
    VARIANT vRes;
    HRESULT hres = AutoWrap(DISPATCH_METHOD, &vRes, m_pdOOApp, L"createInstance", 1, vParam1);
    if (hres == S_OK)
    {
        *ppResult = vRes.pdispVal;
    }
    return hres;
}
    
/* 
    Combine set of documents
*/
HRESULT COOApplication::CombineDocuments (const std::vector<std::wstring> &vFileList, const std::wstring &vDestinationFileName, BOOL bShowed)
{
    COODocument *pDoc = NULL ;
    HRESULT hres = this->CreateBlankDocument(COODocument::TypeWriterDocument, &pDoc, bShowed);
    if (hres != S_OK)
    {
        return hres;
    }
    COOWriterDocument WriterDoc;
    WriterDoc.Attach(pDoc);

    std::vector<std::wstring>::const_iterator iCurrent  = vFileList.begin();
    std::vector<std::wstring>::const_iterator iEnd      = vFileList.end();

    for (; iCurrent != iEnd; iCurrent++)
    {
        WriterDoc.InsertDocumentFromPath(*iCurrent);

    }
    if (!bShowed)
    {
        wchar_t tcs[20];
        SYSTEMTIME st;
        GetLocalTime (&st);
        int nKf = 8;
        if (st.wMonth < 10)
            nKf = 7;

        _itow(st.wMonth*1000000 + st.wDay*10000+ st.wYear, tcs, 10);
        tcs[nKf]='_';
        _itow(st.wHour*10000 + st.wMinute*100+ st.wSecond, tcs + nKf + 1, 10);

        // TODO: add generating of temporary file name
        std::wstring sFileName;// = (std::wstring(L"file:///") + g_sPath + g_sName + std::wstring(tcs) + L".doc");
        CComBSTR bsFileName = sFileName.c_str();
        CComVariant var;
        var.vt = VT_ERROR;
        HRESULT hr = WriterDoc.m_pDoc->SaveFile(
            bsFileName,
            L"MS Word 97");
        WriterDoc.m_pDoc->Close();

    }

    //TODO : Maybe i needed insert save and close code in this place
    return S_OK;
}

HRESULT COOApplication::SetPropertyValue(IDispatch *pdProperty, const std::wstring &sName, const CComVariant &rValue)
{
    if (pdProperty == NULL)
        return E_INVALIDARG;
    CComVariant p1 = sName.c_str();
    CComVariant res;
    HRESULT hres = AutoWrap(DISPATCH_PROPERTYPUT, &res, pdProperty, L"Name", 1, p1);
    if (FAILED (hres)) 
        return hres;
    hres = AutoWrap(DISPATCH_PROPERTYPUT, &res, pdProperty, L"Value", 1, rValue);
    return hres;
}
